package com.example.android.actionbar;

import android.app.ActionBar;
import android.app.Activity;
import android.app.Fragment;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.util.SparseArray;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.WebView;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.SearchView;

public class MainActivity extends Activity {
	private static final String TAG = "ActionBar Example";
	private static final String STATE_CURRENT_FRAGMENT_ID = "current_fragment_id";
	private final SparseArray<Fragment> mFragments = new SparseArray<Fragment>();
	private EditText mEditTextMarkdown;
	private boolean mIsHomeButtonEnabled;
	private boolean mIsHomeAsUpEnabled;
	private int mCurrentFragmentId = R.id.action_edit;
	private Menu mMenu;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
	}

	@Override
	public void onResume() {
		super.onResume();
		setFragment(mCurrentFragmentId);
	}

	@Override
	public void onRestoreInstanceState(Bundle savedInstanceState) {
		setFragment(savedInstanceState.getInt(STATE_CURRENT_FRAGMENT_ID));
	}

	@Override
	public void onSaveInstanceState(Bundle outState) {
		outState.putInt(STATE_CURRENT_FRAGMENT_ID, mCurrentFragmentId);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.main, menu);
		mMenu = menu;
		return true;
	}

	@Override
	public boolean onMenuItemSelected(int featureId, MenuItem item) {
		switch (item.getItemId()) {
		case R.id.action_edit:
		case R.id.action_preview:
		case R.id.action_settings:
			setFragment(item.getItemId());
			return true;
		case android.R.id.home:
			if (mIsHomeButtonEnabled) {
				setFragment(R.id.action_edit);
			} else {
				int id = 0;
				if (mCurrentFragmentId == R.id.action_settings) {
					id = R.id.action_preview;
				} else if (mCurrentFragmentId == R.id.action_preview) {
					id = R.id.action_edit;
				}
				if (id != 0) {
					setFragment(id);
				}
			}
			return true;
		case R.id.action_save: {
			save();
			return true;
		}
		case R.id.toggleButtonSearchViewUsed: {
			Log.d(TAG, "Search was clicked");
			break;
		}
		}
		return super.onMenuItemSelected(featureId, item);
	}

	public void onClick(View view) {
		switch (view.getId()) {
		case R.id.toggleButtonActionBarVisible: {
			ActionBar actionBar = getActionBar();
			if (actionBar.isShowing()) {
				actionBar.hide();
			} else {
				actionBar.show();
			}
			break;
		}
		case R.id.toggleButtonHomeButtonEnabled: {
			toggleHomeButtonEnabled();
			break;
		}
		case R.id.toggleButtonHomeAsUpEnabled: {
			toggleHomeAsUpEnabled();
			break;
		}
		case R.id.toggleButtonSearchViewUsed: {
			toggleSearchView();
			break;
		}
		case R.id.buttonAddItem: {
			mMenu.add("Added")
				.setIcon(android.R.drawable.ic_menu_help)
				.setOnMenuItemClickListener(new MenuItem.OnMenuItemClickListener() {
					@Override
					public boolean onMenuItemClick(MenuItem item) {
						mMenu.removeItem(item.getItemId());
						return true;
					}
				})
				.setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM | MenuItem.SHOW_AS_ACTION_WITH_TEXT);
			break;
		}
		}
	}

	private void setFragment(int id) {
		if (id == android.R.id.home) {
			id = R.id.action_edit;
		}
		Fragment fragment = mFragments.get(id);
		if (fragment == null) {
			fragment = new SectionFragment();
			Bundle args = new Bundle();
			args.putInt(SectionFragment.ARG_FRAGMENT_NUMBER, id);
			fragment.setArguments(args);
			mFragments.put(id, fragment);
		}
		if (id == R.id.action_preview) {
			parseMarkdown(fragment.getView(), mEditTextMarkdown);
		}
		mCurrentFragmentId = id;
		getFragmentManager().beginTransaction().replace(R.id.container, fragment).commit();
	}

	private void save() {
		AsyncTask<Void, Integer, Void> task = new AsyncTask<Void, Integer, Void>() {
			private MenuItem mMenuItem;
			private ProgressBar mProgressBar;
			@Override
			protected void onPreExecute() {
				mMenuItem = mMenu.findItem(R.id.action_save);
				mProgressBar = (ProgressBar) mMenuItem.getActionView();
			}

			@Override
			protected void onProgressUpdate(Integer... values) {
				mProgressBar.setProgress(values[0]);
			}

			@Override
			protected void onPostExecute(Void result) {
				mMenuItem.collapseActionView();
				mProgressBar.setProgress(0);
			}

			@Override
			protected Void doInBackground(Void... params) {
				try {
					for (int i = 0; i < 10; i++) {
						Thread.sleep(200);
						publishProgress(i * 10);
					}
				} catch (InterruptedException e) {
					Log.d(TAG, "InterruptedException in doInBackground", e);
				}
				return null;
			}
		};
		task.execute();
	}

	private void toggleHomeButtonEnabled() {
		getActionBar().setHomeButtonEnabled((mIsHomeButtonEnabled = !mIsHomeButtonEnabled));
	}

	private void toggleHomeAsUpEnabled() {
		ActionBar actionBar = getActionBar();
		actionBar.setDisplayHomeAsUpEnabled((mIsHomeAsUpEnabled = !mIsHomeAsUpEnabled));
		if (!mIsHomeAsUpEnabled && !mIsHomeButtonEnabled) {
			actionBar.setHomeButtonEnabled(false);
		}
	}

	private void toggleSearchView() {
		int id = R.id.toggleButtonSearchViewUsed;
		if (mMenu.findItem(id) == null) {
			mMenu.add(Menu.NONE, id, Menu.NONE, R.string.action_search)
				.setIcon(android.R.drawable.ic_menu_search)
				.setActionView(new SearchView(this))
				.setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS | MenuItem.SHOW_AS_ACTION_WITH_TEXT | MenuItem.SHOW_AS_ACTION_COLLAPSE_ACTION_VIEW);
		} else {
			mMenu.removeItem(id);
		}
	}

	private static void parseMarkdown(View rootView, EditText edit) {
		if (rootView == null) {
			return;
		}
		WebView view = (WebView) rootView.findViewById(R.id.webViewPreview);
		char[] md = edit.getText().toString().toCharArray();
		StringBuilder sb = new StringBuilder();
		for (int i = 0, n; i < md.length; i++) {
			if (md[i] == '#') {
				sb.append("<h");
				for (n = 0; i < md.length && md[i] == '#'; i++, n++);
				sb.append(n + ">");
				for (; i < md.length && md[i] == ' '; i++);
				for (; i < md.length && md[i] != '\n'; sb.append(md[i]), i++);
				sb.append("</h" + n + ">");
			} else if (md[i] == '-') {
				sb.append("<ul>");
ul:				while (i < md.length) {
					sb.append("<li>");
li:					while (i < md.length) {
						for (i += 1; i < md.length && md[i] == ' '; i++);
						for (; i < md.length && md[i] != '\n'; sb.append(md[i]), i++);
						if (i + 1 < md.length)
							if (md[i + 1] == '\n')
								break ul;
							else if (md[i + 1] == '-') {
								i++;
								break li;
							} else
								sb.append(" ");
						else
							i++;
					}
				}
				sb.append("</ul>");
			} else
				if (md[i] == '\n' && i + 1 < md.length) {
					n = i;
					while (i + 1 < md.length && md[i + 1] == '\n')
						i++;
					if (n != i)
						sb.append("<br><br>");
				} else
					sb.append(md[i]);
		}
		view.loadData(new String(sb), "text/html", "UTF-8");
	}

	public static class SectionFragment extends Fragment {
		public static final String ARG_FRAGMENT_NUMBER = "fragment_number";

		@Override
		public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
			int fragmentNumber = getArguments().getInt(ARG_FRAGMENT_NUMBER);
			MainActivity activity = (MainActivity) getActivity();
			View rootView = null;
			switch (fragmentNumber) {
			case R.id.action_edit:
				rootView = inflater.inflate(R.layout.fragment_edit, container, false);
				activity.mEditTextMarkdown = (EditText) rootView.findViewById(R.id.editTextMarkdown);
				break;
			case R.id.action_preview:
				rootView = inflater.inflate(R.layout.fragment_preview, container, false);
				parseMarkdown(rootView, activity.mEditTextMarkdown);
				break;
			case R.id.action_settings:
				rootView = inflater.inflate(R.layout.fragment_settings, container, false);
				break;
			}
			return rootView;
		}
	}
}
